from distutils.core import setup
# To use a consistent encoding

setup(
    name='yson',

    version='0.1',
    packages=['yson'],

    # The project's main homepage.
    url='https://github.com/yydai/yson',
    author='yydai',
    author_email='dai92817@icloud.com',

    license='MIT',
    classifiers=[
        'Programming Language :: Python :: 2.7'
    ],
    keywords=['json', 'parser', 'demo']
)
